@@ -32,7 +32,6 @@ static Env* GetMockEnv() {
   static std::unique_ptr<Env> mock_env(MockEnv::Create(Env::Default()));
   return mock_env.get();
 }
-#ifndef ROCKSDB_LITE
 static Env* NewTestEncryptedEnv(Env* base, const std::string& provider_id) {
   ConfigOptions config_opts;
   config_opts.invoke_prepare_options = false;
@@ -81,7 +80,6 @@ static Env* GetTestFS() {
   EXPECT_NE(fs_env, nullptr);
   return fs_env;
 }
-#endif  // ROCKSDB_LITE
 
 }  // namespace
 class EnvBasicTestWithParam
@@ -111,7 +109,6 @@ INSTANTIATE_TEST_CASE_P(EnvDefault, EnvMoreTestWithParam,
 INSTANTIATE_TEST_CASE_P(MockEnv, EnvBasicTestWithParam,
                         ::testing::Values(&GetMockEnv));
 
-#ifndef ROCKSDB_LITE
 // next statements run env test against default encryption code.
 INSTANTIATE_TEST_CASE_P(EncryptedEnv, EnvBasicTestWithParam,
                         ::testing::Values(&GetCtrEncryptedEnv));
@@ -148,7 +145,6 @@ INSTANTIATE_TEST_CASE_P(CustomEnv, EnvBasicTestWithParam,
 
 INSTANTIATE_TEST_CASE_P(CustomEnv, EnvMoreTestWithParam,
                         ::testing::ValuesIn(GetCustomEnvs()));
-#endif  // ROCKSDB_LITE
 
 TEST_P(EnvBasicTestWithParam, Basics) {
   uint64_t file_size;
@@ -305,7 +301,7 @@ TEST_P(EnvBasicTestWithParam, LargeWrite) {
     read += result.size();
   }
   ASSERT_TRUE(write_data == read_data);
-  delete [] scratch;
+  delete[] scratch;
 }
 
 TEST_P(EnvMoreTestWithParam, GetModTime) {
