@@ -81,13 +81,12 @@ void FilePrefetchBuffer::CalculateOffsetAndLen(size_t alignment,
 
 Status FilePrefetchBuffer::Read(const IOOptions& opts,
                                 RandomAccessFileReader* reader,
-                                Env::IOPriority rate_limiter_priority,
                                 uint64_t read_len, uint64_t chunk_len,
                                 uint64_t rounddown_start, uint32_t index) {
   Slice result;
   Status s = reader->Read(opts, rounddown_start + chunk_len, read_len, &result,
                           bufs_[index].buffer_.BufferStart() + chunk_len,
-                          /*aligned_buf=*/nullptr, rate_limiter_priority);
+                          /*aligned_buf=*/nullptr);
 #ifndef NDEBUG
   if (result.size() < read_len) {
     // Fake an IO error to force db_stress fault injection to ignore
@@ -134,8 +133,7 @@ Status FilePrefetchBuffer::ReadAsync(const IOOptions& opts,
 
 Status FilePrefetchBuffer::Prefetch(const IOOptions& opts,
                                     RandomAccessFileReader* reader,
-                                    uint64_t offset, size_t n,
-                                    Env::IOPriority rate_limiter_priority) {
+                                    uint64_t offset, size_t n) {
   if (!enable_ || reader == nullptr) {
     return Status::OK();
   }
@@ -160,8 +158,8 @@ Status FilePrefetchBuffer::Prefetch(const IOOptions& opts,
                         true /*refit_tail*/, chunk_len);
   size_t read_len = static_cast<size_t>(roundup_len - chunk_len);
 
-  Status s = Read(opts, reader, rate_limiter_priority, read_len, chunk_len,
-                  rounddown_offset, curr_);
+  Status s = Read(opts, reader, read_len, chunk_len, rounddown_offset, curr_);
+
   if (usage_ == FilePrefetchBufferUsage::kTableOpenPrefetchTail && s.ok()) {
     RecordInHistogram(stats_, TABLE_OPEN_PREFETCH_TAIL_READ_BYTES, read_len);
   }
@@ -328,8 +326,7 @@ void FilePrefetchBuffer::PollAndUpdateBuffersIfNeeded(uint64_t offset) {
 
 Status FilePrefetchBuffer::HandleOverlappingData(
     const IOOptions& opts, RandomAccessFileReader* reader, uint64_t offset,
-    size_t length, size_t readahead_size,
-    Env::IOPriority /*rate_limiter_priority*/, bool& copy_to_third_buffer,
+    size_t length, size_t readahead_size, bool& copy_to_third_buffer,
     uint64_t& tmp_offset, size_t& tmp_length) {
   Status s;
   size_t alignment = reader->file()->GetRequiredBufferAlignment();
@@ -412,10 +409,11 @@ Status FilePrefetchBuffer::HandleOverlappingData(
 //        curr_, send async request on curr_, wait for poll to fill second
 //        buffer (if any), and copy remaining data from second buffer to third
 //        buffer.
-Status FilePrefetchBuffer::PrefetchAsyncInternal(
-    const IOOptions& opts, RandomAccessFileReader* reader, uint64_t offset,
-    size_t length, size_t readahead_size, Env::IOPriority rate_limiter_priority,
-    bool& copy_to_third_buffer) {
+Status FilePrefetchBuffer::PrefetchAsyncInternal(const IOOptions& opts,
+                                                 RandomAccessFileReader* reader,
+                                                 uint64_t offset, size_t length,
+                                                 size_t readahead_size,
+                                                 bool& copy_to_third_buffer) {
   if (!enable_) {
     return Status::OK();
   }
@@ -442,8 +440,7 @@ Status FilePrefetchBuffer::PrefetchAsyncInternal(
   //   - switch buffers and curr_ now points to second buffer to copy remaining
   //     data.
   s = HandleOverlappingData(opts, reader, offset, length, readahead_size,
-                            rate_limiter_priority, copy_to_third_buffer,
-                            tmp_offset, tmp_length);
+                            copy_to_third_buffer, tmp_offset, tmp_length);
   if (!s.ok()) {
     return s;
   }
@@ -548,7 +545,9 @@ Status FilePrefetchBuffer::PrefetchAsyncInternal(
     assert(roundup_len1 >= chunk_len1);
     read_len1 = static_cast<size_t>(roundup_len1 - chunk_len1);
   }
-  {
+
+  // Prefetch in second buffer only if readahead_size_ > 0.
+  if (readahead_size_ > 0) {
     // offset and size alignment for second buffer for asynchronous
     // prefetching
     uint64_t rounddown_start2 = roundup_end1;
@@ -581,8 +580,7 @@ Status FilePrefetchBuffer::PrefetchAsyncInternal(
   }
 
   if (read_len1 > 0) {
-    s = Read(opts, reader, rate_limiter_priority, read_len1, chunk_len1,
-             rounddown_start1, curr_);
+    s = Read(opts, reader, read_len1, chunk_len1, rounddown_start1, curr_);
     if (!s.ok()) {
       if (bufs_[second].io_handle_ != nullptr) {
         std::vector<void*> handles;
@@ -610,10 +608,9 @@ bool FilePrefetchBuffer::TryReadFromCache(const IOOptions& opts,
                                           RandomAccessFileReader* reader,
                                           uint64_t offset, size_t n,
                                           Slice* result, Status* status,
-                                          Env::IOPriority rate_limiter_priority,
                                           bool for_compaction /* = false */) {
   bool ret = TryReadFromCacheUntracked(opts, reader, offset, n, result, status,
-                                       rate_limiter_priority, for_compaction);
+                                       for_compaction);
   if (usage_ == FilePrefetchBufferUsage::kTableOpenPrefetchTail && enable_) {
     if (ret) {
       RecordTick(stats_, TABLE_OPEN_PREFETCH_TAIL_HIT);
@@ -627,7 +624,7 @@ bool FilePrefetchBuffer::TryReadFromCache(const IOOptions& opts,
 bool FilePrefetchBuffer::TryReadFromCacheUntracked(
     const IOOptions& opts, RandomAccessFileReader* reader, uint64_t offset,
     size_t n, Slice* result, Status* status,
-    Env::IOPriority rate_limiter_priority, bool for_compaction /* = false */) {
+    bool for_compaction /* = false */) {
   if (track_min_offset_ && offset < min_offset_read_) {
     min_offset_read_ = static_cast<size_t>(offset);
   }
@@ -647,8 +644,7 @@ bool FilePrefetchBuffer::TryReadFromCacheUntracked(
       assert(reader != nullptr);
       assert(max_readahead_size_ >= readahead_size_);
       if (for_compaction) {
-        s = Prefetch(opts, reader, offset, std::max(n, readahead_size_),
-                     rate_limiter_priority);
+        s = Prefetch(opts, reader, offset, std::max(n, readahead_size_));
       } else {
         if (implicit_auto_readahead_) {
           if (!IsEligibleForPrefetch(offset, n)) {
@@ -657,8 +653,8 @@ bool FilePrefetchBuffer::TryReadFromCacheUntracked(
             return false;
           }
         }
-        s = Prefetch(opts, reader, offset, n + readahead_size_,
-                     rate_limiter_priority);
+        UpdateReadAheadSizeForUpperBound(offset, n);
+        s = Prefetch(opts, reader, offset, n + readahead_size_);
       }
       if (!s.ok()) {
         if (status) {
@@ -681,12 +677,12 @@ bool FilePrefetchBuffer::TryReadFromCacheUntracked(
   return true;
 }
 
-bool FilePrefetchBuffer::TryReadFromCacheAsync(
-    const IOOptions& opts, RandomAccessFileReader* reader, uint64_t offset,
-    size_t n, Slice* result, Status* status,
-    Env::IOPriority rate_limiter_priority) {
-  bool ret = TryReadFromCacheAsyncUntracked(opts, reader, offset, n, result,
-                                            status, rate_limiter_priority);
+bool FilePrefetchBuffer::TryReadFromCacheAsync(const IOOptions& opts,
+                                               RandomAccessFileReader* reader,
+                                               uint64_t offset, size_t n,
+                                               Slice* result, Status* status) {
+  bool ret =
+      TryReadFromCacheAsyncUntracked(opts, reader, offset, n, result, status);
   if (usage_ == FilePrefetchBufferUsage::kTableOpenPrefetchTail && enable_) {
     if (ret) {
       RecordTick(stats_, TABLE_OPEN_PREFETCH_TAIL_HIT);
@@ -699,8 +695,7 @@ bool FilePrefetchBuffer::TryReadFromCacheAsync(
 
 bool FilePrefetchBuffer::TryReadFromCacheAsyncUntracked(
     const IOOptions& opts, RandomAccessFileReader* reader, uint64_t offset,
-    size_t n, Slice* result, Status* status,
-    Env::IOPriority rate_limiter_priority) {
+    size_t n, Slice* result, Status* status) {
   if (track_min_offset_ && offset < min_offset_read_) {
     min_offset_read_ = static_cast<size_t>(offset);
   }
@@ -740,7 +735,9 @@ bool FilePrefetchBuffer::TryReadFromCacheAsyncUntracked(
       (bufs_[curr_].async_read_in_progress_ ||
        offset + n >
            bufs_[curr_].offset_ + bufs_[curr_].buffer_.CurrentSize())) {
-    if (readahead_size_ > 0) {
+    // In case readahead_size is trimmed (=0), we still want to poll the data
+    // submitted with explicit_prefetch_submitted_=true.
+    if (readahead_size_ > 0 || explicit_prefetch_submitted_) {
       Status s;
       assert(reader != nullptr);
       assert(max_readahead_size_ >= readahead_size_);
@@ -752,10 +749,13 @@ bool FilePrefetchBuffer::TryReadFromCacheAsyncUntracked(
           return false;
         }
       }
+
+      UpdateReadAheadSizeForUpperBound(offset, n);
+
       // Prefetch n + readahead_size_/2 synchronously as remaining
       // readahead_size_/2 will be prefetched asynchronously.
       s = PrefetchAsyncInternal(opts, reader, offset, n, readahead_size_ / 2,
-                                rate_limiter_priority, copy_to_third_buffer);
+                                copy_to_third_buffer);
       explicit_prefetch_submitted_ = false;
       if (!s.ok()) {
         if (status) {
@@ -829,10 +829,12 @@ Status FilePrefetchBuffer::PrefetchAsync(const IOOptions& opts,
   num_file_reads_ = 0;
   explicit_prefetch_submitted_ = false;
   bool is_eligible_for_prefetching = false;
+
+  UpdateReadAheadSizeForUpperBound(offset, n);
   if (readahead_size_ > 0 &&
       (!implicit_auto_readahead_ ||
-       num_file_reads_ + 1 >= num_file_reads_for_auto_readahead_)) {
-    is_eligible_for_prefetching = true;
+       num_file_reads_ >= num_file_reads_for_auto_readahead_)) {
+      is_eligible_for_prefetching = true;
   }
 
   // 1. Cancel any pending async read to make code simpler as buffers can be out
@@ -894,18 +896,24 @@ Status FilePrefetchBuffer::PrefetchAsync(const IOOptions& opts,
   //   - prefetch_size on second.
   // Calculate length and offsets for reading.
   if (!DoesBufferContainData(curr_)) {
+    uint64_t roundup_len1;
     // Prefetch full data + prefetch_size in curr_.
-    rounddown_start1 = Rounddown(offset_to_read, alignment);
-    roundup_end1 = Roundup(offset_to_read + n + prefetch_size, alignment);
-    uint64_t roundup_len1 = roundup_end1 - rounddown_start1;
-    assert(roundup_len1 >= alignment);
-    assert(roundup_len1 % alignment == 0);
-
+    if (is_eligible_for_prefetching || reader->use_direct_io()) {
+      rounddown_start1 = Rounddown(offset_to_read, alignment);
+      roundup_end1 = Roundup(offset_to_read + n + prefetch_size, alignment);
+      roundup_len1 = roundup_end1 - rounddown_start1;
+      assert(roundup_len1 >= alignment);
+      assert(roundup_len1 % alignment == 0);
+    } else {
+      rounddown_start1 = offset_to_read;
+      roundup_end1 = offset_to_read + n;
+      roundup_len1 = roundup_end1 - rounddown_start1;
+    }
     CalculateOffsetAndLen(alignment, rounddown_start1, roundup_len1, curr_,
                           false, chunk_len1);
     assert(chunk_len1 == 0);
     assert(roundup_len1 >= chunk_len1);
-    read_len1 = static_cast<size_t>(roundup_len1 - chunk_len1);
+    read_len1 = static_cast<size_t>(roundup_len1);
     bufs_[curr_].offset_ = rounddown_start1;
   }
 
@@ -941,6 +949,7 @@ Status FilePrefetchBuffer::PrefetchAsync(const IOOptions& opts,
     prev_len_ = 0;
   }
   if (read_len2) {
+    TEST_SYNC_POINT("FilePrefetchBuffer::PrefetchAsync:ExtraPrefetching");
     s = ReadAsync(opts, reader, read_len2, rounddown_start2, second);
     if (!s.ok()) {
       DestroyAndClearIOHandle(second);
