@@ -79,7 +79,8 @@ class SecondaryCache : public Customizable {
   // and writes it to this cache tier. OK may be returned even if the insertion
   // is not made.
   virtual Status Insert(const Slice& key, Cache::ObjectPtr obj,
-                        const Cache::CacheItemHelper* helper) = 0;
+                        const Cache::CacheItemHelper* helper,
+                        bool force_insert) = 0;
 
   // Insert a value from its saved/persistable data (typically uncompressed
   // block), as if generated by SaveToCallback/SizeCallback. This can be used
@@ -134,6 +135,17 @@ class SecondaryCache : public Customizable {
   virtual Status GetCapacity(size_t& /* capacity */) {
     return Status::NotSupported();
   }
+
+  // Temporarily decrease the cache capacity in RAM by the specified amount.
+  // The caller should call Inflate() to restore the cache capacity. This is
+  // intended to be lighter weight than SetCapacity(). The latter evenly
+  // distributes the new capacity across all shards and is meant for large
+  // changes in capacity, whereas the former is meant for relatively small
+  // changes and may be uneven by lowering capacity in a single shard.
+  virtual Status Deflate(size_t /*decrease*/) { return Status::NotSupported(); }
+
+  // Restore the capacity reduced by a prior call to Deflate().
+  virtual Status Inflate(size_t /*increase*/) { return Status::NotSupported(); }
 };
 
 }  // namespace ROCKSDB_NAMESPACE
