@@ -53,8 +53,11 @@
 #include <zdict.h>
 #endif  // ZSTD_VERSION_NUMBER >= 10103
 // v1.4.0+
+// ZSTD_Compress2(), ZSTD_compressStream2() and frame parameters all belong to
+// advanced APIs and require v1.4.0+.
+// https://github.com/facebook/zstd/blob/eb9f881eb810f2242f1ef36b3f3e7014eecb8fa6/lib/zstd.h#L297C40-L297C45
 #if ZSTD_VERSION_NUMBER >= 10400
-#define ZSTD_STREAMING
+#define ZSTD_ADVANCED
 #endif  // ZSTD_VERSION_NUMBER >= 10400
 namespace ROCKSDB_NAMESPACE {
 // Need this for the context allocation override
@@ -180,6 +183,9 @@ struct CompressionDict {
       if (level == CompressionOptions::kDefaultCompressionLevel) {
         // 3 is the value of ZSTD_CLEVEL_DEFAULT (not exposed publicly), see
         // https://github.com/facebook/zstd/issues/1148
+        // TODO(cbi): ZSTD_CLEVEL_DEFAULT is exposed after
+        //  https://github.com/facebook/zstd/pull/1174. Use ZSTD_CLEVEL_DEFAULT
+        //  instead of hardcoding 3.
         level = 3;
       }
       // Should be safe (but slower) if below call fails as we'll use the
@@ -363,14 +369,43 @@ class CompressionContext {
  private:
 #if defined(ZSTD) && (ZSTD_VERSION_NUMBER >= 500)
   ZSTD_CCtx* zstd_ctx_ = nullptr;
-  void CreateNativeContext(CompressionType type) {
-    if (type == kZSTD || type == kZSTDNotFinalCompression) {
+
+  ZSTD_CCtx* CreateZSTDContext() {
 #ifdef ROCKSDB_ZSTD_CUSTOM_MEM
-      zstd_ctx_ =
-          ZSTD_createCCtx_advanced(port::GetJeZstdAllocationOverrides());
+    return ZSTD_createCCtx_advanced(port::GetJeZstdAllocationOverrides());
 #else   // ROCKSDB_ZSTD_CUSTOM_MEM
-      zstd_ctx_ = ZSTD_createCCtx();
+    return ZSTD_createCCtx();
 #endif  // ROCKSDB_ZSTD_CUSTOM_MEM
+  }
+
+  void CreateNativeContext(CompressionType type, int level, bool checksum) {
+    if (type == kZSTD || type == kZSTDNotFinalCompression) {
+      zstd_ctx_ = CreateZSTDContext();
+#ifdef ZSTD_ADVANCED
+      if (level == CompressionOptions::kDefaultCompressionLevel) {
+        // 3 is the value of ZSTD_CLEVEL_DEFAULT (not exposed publicly), see
+        // https://github.com/facebook/zstd/issues/1148
+        level = 3;
+      }
+      size_t err =
+          ZSTD_CCtx_setParameter(zstd_ctx_, ZSTD_c_compressionLevel, level);
+      if (ZSTD_isError(err)) {
+        assert(false);
+        ZSTD_freeCCtx(zstd_ctx_);
+        zstd_ctx_ = CreateZSTDContext();
+      }
+      if (checksum) {
+        err = ZSTD_CCtx_setParameter(zstd_ctx_, ZSTD_c_checksumFlag, 1);
+        if (ZSTD_isError(err)) {
+          assert(false);
+          ZSTD_freeCCtx(zstd_ctx_);
+          zstd_ctx_ = CreateZSTDContext();
+        }
+      }
+#else
+      (void)level;
+      (void)checksum;
+#endif
     }
   }
   void DestroyNativeContext() {
@@ -388,12 +423,14 @@ class CompressionContext {
 
 #else   // ZSTD && (ZSTD_VERSION_NUMBER >= 500)
  private:
-  void CreateNativeContext(CompressionType /* type */) {}
+  void CreateNativeContext(CompressionType /* type */, int /* level */,
+                           bool /* checksum */) {}
   void DestroyNativeContext() {}
 #endif  // ZSTD && (ZSTD_VERSION_NUMBER >= 500)
  public:
-  explicit CompressionContext(CompressionType type) {
-    CreateNativeContext(type);
+  explicit CompressionContext(CompressionType type,
+                              const CompressionOptions& options) {
+    CreateNativeContext(type, options.level, options.checksum);
   }
   ~CompressionContext() { DestroyNativeContext(); }
   CompressionContext(const CompressionContext&) = delete;
@@ -525,7 +562,7 @@ inline bool ZSTDNotFinal_Supported() {
 }
 
 inline bool ZSTD_Streaming_Supported() {
-#if defined(ZSTD) && defined(ZSTD_STREAMING)
+#if defined(ZSTD_ADVANCED)
   return true;
 #else
   return false;
@@ -1343,30 +1380,44 @@ inline bool ZSTD_Compress(const CompressionInfo& info, const char* input,
   size_t compressBound = ZSTD_compressBound(length);
   output->resize(static_cast<size_t>(output_header_len + compressBound));
   size_t outlen = 0;
-  int level;
-  if (info.options().level == CompressionOptions::kDefaultCompressionLevel) {
-    // 3 is the value of ZSTD_CLEVEL_DEFAULT (not exposed publicly), see
-    // https://github.com/facebook/zstd/issues/1148
-    level = 3;
-  } else {
-    level = info.options().level;
-  }
 #if ZSTD_VERSION_NUMBER >= 500  // v0.5.0+
   ZSTD_CCtx* context = info.context().ZSTDPreallocCtx();
   assert(context != nullptr);
+#ifdef ZSTD_ADVANCED
+  if (info.dict().GetDigestedZstdCDict() != nullptr) {
+    ZSTD_CCtx_refCDict(context, info.dict().GetDigestedZstdCDict());
+  } else {
+    ZSTD_CCtx_loadDictionary(context, info.dict().GetRawDict().data(),
+                             info.dict().GetRawDict().size());
+  }
+
+  // Compression level is set in `contex` during CreateNativeContext()
+  outlen = ZSTD_compress2(context, &(*output)[output_header_len], compressBound,
+                          input, length);
+#else                           // ZSTD_ADVANCED
 #if ZSTD_VERSION_NUMBER >= 700  // v0.7.0+
   if (info.dict().GetDigestedZstdCDict() != nullptr) {
     outlen = ZSTD_compress_usingCDict(context, &(*output)[output_header_len],
                                       compressBound, input, length,
                                       info.dict().GetDigestedZstdCDict());
   }
-#endif  // ZSTD_VERSION_NUMBER >= 700
+#endif                          // ZSTD_VERSION_NUMBER >= 700
+  // TODO (cbi): error handling for compression.
   if (outlen == 0) {
+    int level;
+    if (info.options().level == CompressionOptions::kDefaultCompressionLevel) {
+      // 3 is the value of ZSTD_CLEVEL_DEFAULT (not exposed publicly), see
+      // https://github.com/facebook/zstd/issues/1148
+      level = 3;
+    } else {
+      level = info.options().level;
+    }
     outlen = ZSTD_compress_usingDict(context, &(*output)[output_header_len],
                                      compressBound, input, length,
                                      info.dict().GetRawDict().data(),
                                      info.dict().GetRawDict().size(), level);
   }
+#endif                          // ZSTD_ADVANCED
 #else   // up to v0.4.x
   outlen = ZSTD_compress(&(*output)[output_header_len], compressBound, input,
                          length, level);
@@ -1387,17 +1438,28 @@ inline bool ZSTD_Compress(const CompressionInfo& info, const char* input,
 
 // @param compression_dict Data for presetting the compression library's
 //    dictionary.
+// @param error_message If not null, will be set if decompression fails.
+//
+// Returns nullptr if decompression fails.
 inline CacheAllocationPtr ZSTD_Uncompress(
     const UncompressionInfo& info, const char* input_data, size_t input_length,
-    size_t* uncompressed_size, MemoryAllocator* allocator = nullptr) {
+    size_t* uncompressed_size, MemoryAllocator* allocator = nullptr,
+    const char** error_message = nullptr) {
 #ifdef ZSTD
+  static const char* const kErrorDecodeOutputSize =
+      "Cannot decode output size.";
+  static const char* const kErrorOutputLenMismatch =
+      "Decompressed size does not match header.";
   uint32_t output_len = 0;
   if (!compression::GetDecompressedSizeInfo(&input_data, &input_length,
                                             &output_len)) {
+    if (error_message) {
+      *error_message = kErrorDecodeOutputSize;
+    }
     return nullptr;
   }
 
-  auto output = AllocateBlock(output_len, allocator);
+  CacheAllocationPtr output = AllocateBlock(output_len, allocator);
   size_t actual_output_length = 0;
 #if ZSTD_VERSION_NUMBER >= 500  // v0.5.0+
   ZSTD_DCtx* context = info.context().GetZSTDContext();
@@ -1407,19 +1469,31 @@ inline CacheAllocationPtr ZSTD_Uncompress(
     actual_output_length = ZSTD_decompress_usingDDict(
         context, output.get(), output_len, input_data, input_length,
         info.dict().GetDigestedZstdDDict());
-  }
+  } else {
 #endif  // ROCKSDB_ZSTD_DDICT
-  if (actual_output_length == 0) {
     actual_output_length = ZSTD_decompress_usingDict(
         context, output.get(), output_len, input_data, input_length,
         info.dict().GetRawDict().data(), info.dict().GetRawDict().size());
+#ifdef ROCKSDB_ZSTD_DDICT
   }
+#endif  // ROCKSDB_ZSTD_DDICT
 #else   // up to v0.4.x
   (void)info;
   actual_output_length =
       ZSTD_decompress(output.get(), output_len, input_data, input_length);
 #endif  // ZSTD_VERSION_NUMBER >= 500
-  assert(actual_output_length == output_len);
+  if (ZSTD_isError(actual_output_length)) {
+    if (error_message) {
+      *error_message = ZSTD_getErrorName(actual_output_length);
+    }
+    return nullptr;
+  } else if (actual_output_length != output_len) {
+    if (error_message) {
+      *error_message = kErrorOutputLenMismatch;
+    }
+    return nullptr;
+  }
+
   *uncompressed_size = actual_output_length;
   return output;
 #else  // ZSTD
@@ -1428,6 +1502,7 @@ inline CacheAllocationPtr ZSTD_Uncompress(
   (void)input_length;
   (void)uncompressed_size;
   (void)allocator;
+  (void)error_message;
   return nullptr;
 #endif
 }
@@ -1530,6 +1605,7 @@ inline std::string ZSTD_FinalizeDictionary(
     return dict_data;
   }
 #else   // up to v1.4.4
+  assert(false);
   (void)samples;
   (void)sample_lens;
   (void)max_dict_bytes;
@@ -1589,7 +1665,8 @@ inline bool CompressData(const Slice& raw,
 inline CacheAllocationPtr UncompressData(
     const UncompressionInfo& uncompression_info, const char* data, size_t n,
     size_t* uncompressed_size, uint32_t compress_format_version,
-    MemoryAllocator* allocator = nullptr) {
+    MemoryAllocator* allocator = nullptr,
+    const char** error_message = nullptr) {
   switch (uncompression_info.type()) {
     case kSnappyCompression:
       return Snappy_Uncompress(data, n, uncompressed_size, allocator);
@@ -1609,8 +1686,9 @@ inline CacheAllocationPtr UncompressData(
       return CacheAllocationPtr(XPRESS_Uncompress(data, n, uncompressed_size));
     case kZSTD:
     case kZSTDNotFinalCompression:
+      // TODO(cbi): error message handling for other compression algorithms.
       return ZSTD_Uncompress(uncompression_info, data, n, uncompressed_size,
-                             allocator);
+                             allocator, error_message);
     default:
       return CacheAllocationPtr();
   }
@@ -1743,7 +1821,7 @@ class ZSTDStreamingCompress final : public StreamingCompress {
                                  size_t max_output_len)
       : StreamingCompress(kZSTD, opts, compress_format_version,
                           max_output_len) {
-#ifdef ZSTD_STREAMING
+#ifdef ZSTD_ADVANCED
     cctx_ = ZSTD_createCCtx();
     // Each compressed frame will have a checksum
     ZSTD_CCtx_setParameter(cctx_, ZSTD_c_checksumFlag, 1);
@@ -1752,14 +1830,14 @@ class ZSTDStreamingCompress final : public StreamingCompress {
 #endif
   }
   ~ZSTDStreamingCompress() override {
-#ifdef ZSTD_STREAMING
+#ifdef ZSTD_ADVANCED
     ZSTD_freeCCtx(cctx_);
 #endif
   }
   int Compress(const char* input, size_t input_size, char* output,
                size_t* output_pos) override;
   void Reset() override;
-#ifdef ZSTD_STREAMING
+#ifdef ZSTD_ADVANCED
   ZSTD_CCtx* cctx_;
   ZSTD_inBuffer input_buffer_;
 #endif
@@ -1770,14 +1848,14 @@ class ZSTDStreamingUncompress final : public StreamingUncompress {
   explicit ZSTDStreamingUncompress(uint32_t compress_format_version,
                                    size_t max_output_len)
       : StreamingUncompress(kZSTD, compress_format_version, max_output_len) {
-#ifdef ZSTD_STREAMING
+#ifdef ZSTD_ADVANCED
     dctx_ = ZSTD_createDCtx();
     assert(dctx_ != nullptr);
     input_buffer_ = {/*src=*/nullptr, /*size=*/0, /*pos=*/0};
 #endif
   }
   ~ZSTDStreamingUncompress() override {
-#ifdef ZSTD_STREAMING
+#ifdef ZSTD_ADVANCED
     ZSTD_freeDCtx(dctx_);
 #endif
   }
@@ -1786,7 +1864,7 @@ class ZSTDStreamingUncompress final : public StreamingUncompress {
   void Reset() override;
 
  private:
-#ifdef ZSTD_STREAMING
+#ifdef ZSTD_ADVANCED
   ZSTD_DCtx* dctx_;
   ZSTD_inBuffer input_buffer_;
 #endif
