@@ -39,7 +39,14 @@ static OperationInfo global_operation_table[] = {
     {ThreadStatus::OP_UNKNOWN, ""},
     {ThreadStatus::OP_COMPACTION, "Compaction"},
     {ThreadStatus::OP_FLUSH, "Flush"},
-    {ThreadStatus::OP_DBOPEN, "DBOpen"}};
+    {ThreadStatus::OP_DBOPEN, "DBOpen"},
+    {ThreadStatus::OP_GET, "Get"},
+    {ThreadStatus::OP_MULTIGET, "MultiGet"},
+    {ThreadStatus::OP_DBITERATOR, "DBIterator"},
+    {ThreadStatus::OP_VERIFY_DB_CHECKSUM, "VerifyDBChecksum"},
+    {ThreadStatus::OP_VERIFY_FILE_CHECKSUMS, "VerifyFileChecksums"},
+
+};
 
 struct OperationStageInfo {
   const ThreadStatus::OperationStage stage;
